/*
 * Copyright (c) 2016-2018, Sonos, Inc.
 *
 * SPDX-License-Identifier:     GPL-2.0
 *
 * Driver for generic Sonos microphone control.
 */

#ifndef MICCTL_H
#define MICCTL_H

#define MICCTL_MAJOR_NUMBER	(215)
#define MICCTL_DEVICE_NAME	"micctl"
#define MICCTL_DEVICE_NODE	"/dev/"MICCTL_DEVICE_NAME

#define MICCTL_VERSION		(1)

#define MICCTL_GET_VERSION	_IOR(MICCTL_MAJOR_NUMBER, 0, uint32_t)
#define MICCTL_SET_MUTE		_IOW(MICCTL_MAJOR_NUMBER, 1, uint8_t)

struct adc_ops {
	int (*mute)(struct adc_ops *, int);
	int (*is_muted)(struct adc_ops *);
	struct adc_ops *next;
};

#if defined(SONOS_ARCH_ATTR_SOC_IS_IMX6)
#include "pcm1864.h"
#define MICCTL_HW_INITS { \
	pcm1864_init, \
	NULL \
} ;
#define MICCTL_HW_EXITS { \
	pcm1864_exit, \
	NULL \
} ;
#elif defined(SONOS_ARCH_ATTR_SOC_IS_MT8521P)
#include "ice40_fpga.h"
#include "ice40_mics.h"
#define MICCTL_HW_INITS { \
	ice40_fpga_release_reset, \
	ice40_mics_init, \
	NULL \
} ;
#define MICCTL_HW_EXITS { \
	ice40_mics_exit, \
	NULL \
} ;
#elif defined(SONOS_ARCH_ATTR_SOC_IS_A113)
#include "gpio_mics.h"
#define MICCTL_HW_INITS { \
	gpio_mics_init, \
	NULL \
} ;
#define MICCTL_HW_EXITS { \
	gpio_mics_exit, \
	NULL \
} ;
#elif defined(SONOS_ARCH_ATTR_SOC_IS_IMX7ULP)
#include "gpio_mics.h"
#define MICCTL_HW_INITS { \
	gpio_mics_init, \
	NULL \
} ;
#define MICCTL_HW_EXITS { \
	gpio_mics_exit, \
	NULL \
} ;
#else
#error "SOC_IS not defined"
#endif

void micctl_register_callbacks(struct adc_ops *ops);

#endif
