/*
 * Copyright (c) 2016-2017, Sonos, Inc.
 *
 * SPDX-License-Identifier:     GPL-2.0
 *
 * Driver for the enabling/disabling MICs through the iCE40 FPGA device over i2c
 */

#include <linux/device.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/uaccess.h>
#include "blackbox.h"
#include "micctl.h"
#include "ice40_fpga.h"
#include "ice40_mics.h"

#define ICE40_REG_MIC_CONTROL		0xC0
#define ICE40_MSK_MIC_MUTE			BIT(7)
#define ICE40_MSK_MIC_PDM_EN		BIT(6)

struct adc_ops ops;

int ice40_mics_mute(struct adc_ops *ref, int on)
{
	(void)ref;
	ice40_fpga_write_reg(ICE40_REG_MIC_CONTROL, (on ? 0 : ICE40_MSK_MIC_MUTE));
	return 0;
}

int ice40_mics_init(void)
{
	ice40_fpga_write_reg(ICE40_REG_MIC_CONTROL, 0);

	ops.mute	= ice40_mics_mute;
	micctl_register_callbacks(&ops);

	bb_log(BB_MOD_MICCTL, BB_LVL_INFO, "iCE40 micctl driver initialized and callbacks registered");
	return 0;
}

void ice40_mics_exit(void)
{
	if (ice40_mics_mute(&ops, 1)) {
		bb_log(BB_MOD_MICCTL, BB_LVL_ERR, "Failed to trun OFF mics while removing the driver!");
	} else {
		bb_log(BB_MOD_MICCTL, BB_LVL_INFO, "iCE40 micctl driver removed");
	}
}

