/*
 * Copyright (c) 2018 Sonos Inc.
 * All rights reserved.
 */

#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/kallsyms.h>
#include <asm/uaccess.h>
#include "blackbox.h"
#include "power_event_queue.h"

#define POWER_EVENT_REG_PROC_FILENAME "driver/power_event_queue_registrations"

#define case_enum_ret_str(e) \
	case e: \
	if (strchr(#e, '_') == NULL) return #e; \
	return (strchr(#e, '_')+1)

char * power_event_queue_get_source_string(enum PowerEventSource source)
{
	switch(source) {
		case_enum_ret_str(POWER_EVTSOURCE_NO_SOURCE);
		case_enum_ret_str(POWER_EVTSOURCE_BUTTON_PWR);
		case_enum_ret_str(POWER_EVTSOURCE_BLE);
		case_enum_ret_str(POWER_EVTSOURCE_WIFI);
		case_enum_ret_str(POWER_EVTSOURCE_BATTERY);
		case_enum_ret_str(POWER_EVTSOURCE_CHARGER);
		case_enum_ret_str(POWER_EVTSOURCE_RTC);
		case POWER_EVTSOURCE_NUM_SOURCES:
			return "*";
	}
	return "power-source-unknown";
}
EXPORT_SYMBOL(power_event_queue_get_source_string);

char * power_event_queue_get_event_string(enum PowerEventInfo event)
{
	switch (event) {
		case_enum_ret_str(POWER_EVTINFO_NO_EVENT);
		case_enum_ret_str(POWER_EVTINFO_PRESSED);
		case_enum_ret_str(POWER_EVTINFO_RELEASED);
		case_enum_ret_str(POWER_EVTINFO_QUEUE_OVERRUN);
		case_enum_ret_str(POWER_EVTINFO_LOW_BATTERY);
		case_enum_ret_str(POWER_EVTINFO_CRIT_BATTERY);
		case_enum_ret_str(POWER_EVTINFO_ON_BATTERY);
		case_enum_ret_str(POWER_EVTINFO_ON_CHARGER);
		case_enum_ret_str(POWER_EVTINFO_WAKEUP);
		case_enum_ret_str(POWER_EVTINFO_ALARM);
		case POWER_EVTINFO_NUM_EVENTS:
			return "*";
	}
	return "power-event-unknown";
}
EXPORT_SYMBOL(power_event_queue_get_event_string);

static int power_eqr_show(struct seq_file *m, void *v)
{
	struct power_event_reg *eqr;

	down_read(&power_event_queue_reg_lock);
	if (list_empty(&power_event_queue_reg_list)) {
		seq_printf(m, "\nno Power Event Callback registrations exist\n\n");
	} else {
		seq_printf(m, "\nPower Event Callback registrations\n\n");
		seq_printf(m, "%15s%22s%9s %9s %s\n", "Source", "Event", "Param", "Callback", "Name");
		list_for_each_entry(eqr, &power_event_queue_reg_list, list)
		seq_printf(m, "\n");
	}
	up_read(&power_event_queue_reg_lock);
	return 0;
}

static int power_eqr_open(struct inode *inode, struct file *file)
{
	return single_open(file, power_eqr_show, PDE_DATA(inode));
}

static struct file_operations power_eqr_proc_operations = {
	.owner	= THIS_MODULE,
	.open	= power_eqr_open,
	.write	= NULL,
};

int power_event_queue_init_proc(void)
{
	struct proc_dir_entry *power_entry;
	power_entry = proc_create_data(POWER_EVENT_REG_PROC_FILENAME, 0666, NULL, &power_eqr_proc_operations, NULL);
	if (power_entry == NULL) {
		bb_log(BB_MOD_EVENT, BB_LVL_ERR, "power proc init failed\n");
		return(-EIO);
	}
	return 0;
}

void power_event_queue_remove_proc(void)
{
	remove_proc_entry(POWER_EVENT_REG_PROC_FILENAME, NULL);
}
