/*
 * Copyright (c) 2015-2019, Sonos, Inc.
 *
 * SPDX-License-Identifier:     GPL-2.0
 *
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <linux/stddef.h>
#include <linux/string.h>
#include <linux/delay.h>
#include <linux/workqueue.h>
#include <linux/jiffies.h>
#include "blackbox.h"
#include "sdd.h"
#include "captouch_sim.h"

static struct delayed_work	captouch_work;
int captouch_msec_remaining;
unsigned int captouch_msec_interval;
struct captouch_match_entry *captouch_active_entry = NULL;
struct captouch_match_entry  *captouch_match_table;

struct captouch_sequence_entry captouch_ac_swipe_sequence[] = {
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED, 124},
	{EVTSOURCE_CAPZONEB, EVTINFO_SWIPED,   24},
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED, 124},
	{EVTSOURCE_CAPZONEC, EVTINFO_RELEASED,  0},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT}
};

struct captouch_sequence_entry captouch_ca_swipe_sequence[] = {
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED, 172},
	{EVTSOURCE_CAPZONEB, EVTINFO_SWIPED,   68},
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED, 260},
	{EVTSOURCE_CAPZONEA, EVTINFO_RELEASED,  0},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_ab_bad_sequence[] = {
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED, 124},
	{EVTSOURCE_CAPZONEB, EVTINFO_SWIPED,   32},
	{EVTSOURCE_CAPZONEB, EVTINFO_RELEASED,  0},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_cb_bad_sequence[] = {
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED, 172},
	{EVTSOURCE_CAPZONEB, EVTINFO_SWIPED,   68},
	{EVTSOURCE_CAPZONEB, EVTINFO_RELEASED,  0},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_acac_sequence[] = {
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED, 156},
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED,  44},
	{EVTSOURCE_CAPZONEA, EVTINFO_RELEASED,776},
	{EVTSOURCE_CAPZONEC, EVTINFO_RELEASED,544},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_acca_sequence[] = {
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED, 138},
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED,  96},
	{EVTSOURCE_CAPZONEC, EVTINFO_RELEASED, 94},
	{EVTSOURCE_CAPZONEA, EVTINFO_RELEASED, 94},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_caca_sequence[] = {
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED,  444},
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED,  332},
	{EVTSOURCE_CAPZONEC, EVTINFO_RELEASED, 772},
	{EVTSOURCE_CAPZONEA, EVTINFO_RELEASED, 114},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT,  0}
};

struct captouch_sequence_entry captouch_caac_sequence[] = {
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED, 112},
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED,  88},
	{EVTSOURCE_CAPZONEA, EVTINFO_RELEASED, 34},
	{EVTSOURCE_CAPZONEC, EVTINFO_RELEASED, 12},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_cat_sequence[] = {
	{EVTSOURCE_CAPZONEC,   EVTINFO_PRESSED, 142},
	{EVTSOURCE_CAPZONEB,   EVTINFO_SWIPED,   38},
	{EVTSOURCE_CAPZONECAT, EVTINFO_PRESSED, 444},
	{EVTSOURCE_CAPZONEB,   EVTINFO_RELEASED, 12},
	{EVTSOURCE_CAPZONECAT, EVTINFO_RELEASED,  0},
	{EVTSOURCE_NO_SOURCE,  EVTINFO_NO_EVENT,  0}
};

struct captouch_sequence_entry captouch_baab_sequence[] = {
	{EVTSOURCE_CAPZONEB, EVTINFO_PRESSED,  99},
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED,  86},
	{EVTSOURCE_CAPZONEA, EVTINFO_RELEASED, 33},
	{EVTSOURCE_CAPZONEB, EVTINFO_RELEASED, 12},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_cbcb_sequence[] = {
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED, 377},
	{EVTSOURCE_CAPZONEB, EVTINFO_PRESSED, 136},
	{EVTSOURCE_CAPZONEC, EVTINFO_RELEASED, 33},
	{EVTSOURCE_CAPZONEB, EVTINFO_RELEASED, 12},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_match_entry captouch_default_table[] = {
	{.keyword = "zonea", .source = EVTSOURCE_CAPZONEA},
	{.keyword = "zoneb", .source = EVTSOURCE_CAPZONEB},
	{.keyword = "zonec", .source = EVTSOURCE_CAPZONEC},
	{.keyword = "swipe-good-ac", .sequence = captouch_ac_swipe_sequence},
	{.keyword = "swipe-good-ca", .sequence = captouch_ca_swipe_sequence},
	{.keyword = "swipe-bad-ab", .sequence = captouch_ab_bad_sequence},
	{.keyword = "swipe-bad-cb", .sequence = captouch_cb_bad_sequence},
	{.keyword = "multi-acac", .sequence = captouch_acac_sequence},
	{.keyword = "multi-acca", .sequence = captouch_acca_sequence},
	{.keyword = "multi-caca", .sequence = captouch_caca_sequence},
	{.keyword = "multi-caac", .sequence = captouch_caac_sequence},
	{.keyword = "multi-baab", .sequence = captouch_baab_sequence},
	{.keyword = "multi-cbcb", .sequence = captouch_cbcb_sequence},
	{.keyword = "cat", .sequence = captouch_cat_sequence},
	{NULL, NULL}
};

struct captouch_sequence_entry captouch_mabbam_sequence[] = {
	{EVTSOURCE_CAPZONEM, EVTINFO_PRESSED, 312},
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED, 249},
	{EVTSOURCE_CAPZONEB, EVTINFO_PRESSED, 387},
	{EVTSOURCE_CAPZONEB, EVTINFO_RELEASED, 81},
	{EVTSOURCE_CAPZONEA, EVTINFO_RELEASED, 85},
	{EVTSOURCE_CAPZONEM, EVTINFO_RELEASED, 19},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_cmacma_sequence[] = {
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED, 291},
	{EVTSOURCE_CAPZONEM, EVTINFO_PRESSED, 444},
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED, 184},
	{EVTSOURCE_CAPZONEC, EVTINFO_RELEASED, 91},
	{EVTSOURCE_CAPZONEM, EVTINFO_RELEASED, 67},
	{EVTSOURCE_CAPZONEA, EVTINFO_RELEASED, 75},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_bcmmbc_sequence[] = {
	{EVTSOURCE_CAPZONEB, EVTINFO_PRESSED, 287},
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED, 112},
	{EVTSOURCE_CAPZONEM, EVTINFO_PRESSED, 222},
	{EVTSOURCE_CAPZONEM, EVTINFO_RELEASED, 44},
	{EVTSOURCE_CAPZONEB, EVTINFO_RELEASED, 82},
	{EVTSOURCE_CAPZONEC, EVTINFO_RELEASED, 41},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_cmcm_sequence[] = {
	{EVTSOURCE_CAPZONEC, EVTINFO_PRESSED, 111},
	{EVTSOURCE_CAPZONEM, EVTINFO_PRESSED, 124},
	{EVTSOURCE_CAPZONEC, EVTINFO_RELEASED, 94},
	{EVTSOURCE_CAPZONEM, EVTINFO_RELEASED, 33},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_mbbm_sequence[] = {
	{EVTSOURCE_CAPZONEM, EVTINFO_PRESSED, 311},
	{EVTSOURCE_CAPZONEB, EVTINFO_PRESSED, 112},
	{EVTSOURCE_CAPZONEB, EVTINFO_RELEASED, 79},
	{EVTSOURCE_CAPZONEM, EVTINFO_RELEASED, 82},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_sequence_entry captouch_amma_sequence[] = {
	{EVTSOURCE_CAPZONEA, EVTINFO_PRESSED, 212},
	{EVTSOURCE_CAPZONEM, EVTINFO_PRESSED, 187},
	{EVTSOURCE_CAPZONEM, EVTINFO_RELEASED, 93},
	{EVTSOURCE_CAPZONEA, EVTINFO_RELEASED, 12},
	{EVTSOURCE_NO_SOURCE, EVTINFO_NO_EVENT, 0}
};

struct captouch_match_entry captouch_default_zonem_table[] = {
	{.keyword = "zonea", .source = EVTSOURCE_CAPZONEA},
	{.keyword = "zoneb", .source = EVTSOURCE_CAPZONEB},
	{.keyword = "zonec", .source = EVTSOURCE_CAPZONEC},
	{.keyword = "zonem", .source = EVTSOURCE_CAPZONEM},
	{.keyword = "swipe-good-ac", .sequence = captouch_ac_swipe_sequence},
	{.keyword = "swipe-good-ca", .sequence = captouch_ca_swipe_sequence},
	{.keyword = "swipe-bad-ab", .sequence = captouch_ab_bad_sequence},
	{.keyword = "swipe-bad-cb", .sequence = captouch_cb_bad_sequence},
	{.keyword = "multi-acac", .sequence = captouch_acac_sequence},
	{.keyword = "multi-acca", .sequence = captouch_acca_sequence},
	{.keyword = "multi-caca", .sequence = captouch_caca_sequence},
	{.keyword = "multi-caac", .sequence = captouch_caac_sequence},
	{.keyword = "multi-baab", .sequence = captouch_baab_sequence},
	{.keyword = "multi-cbcb", .sequence = captouch_cbcb_sequence},
	{.keyword = "cat", .sequence = captouch_cat_sequence},
	{.keyword = "multi-mabbam", .sequence = captouch_mabbam_sequence},
	{.keyword = "multi-cmacma", .sequence = captouch_cmacma_sequence},
	{.keyword = "multi-bcmmbc", .sequence = captouch_bcmmbc_sequence},
	{.keyword = "multi-cmcm", .sequence = captouch_cmcm_sequence},
	{.keyword = "multi-mbbm", .sequence = captouch_mbbm_sequence},
	{.keyword = "multi-amma", .sequence = captouch_amma_sequence},
	{NULL, NULL}
};

struct captouch_match_entry captouch_empty_table[] = {
	{NULL, NULL}
};

void captouch_sim(struct work_struct *d)
{
	enum EventInfo info;

	captouch_msec_remaining -= captouch_msec_interval;
	info = (captouch_msec_remaining <= 0) ? EVTINFO_RELEASED : EVTINFO_REPEATED;
	event_queue_send_event(captouch_active_entry->source, info);

	if (captouch_msec_remaining > 0) {
		captouch_msec_interval = (captouch_msec_remaining > SGE_REPEAT_TIME) ? SGE_REPEAT_TIME : captouch_msec_remaining;
		schedule_delayed_work(&captouch_work, msecs_to_jiffies(captouch_msec_interval));
	} else {
		captouch_active_entry = NULL;
		sdd_set_grr(EVTSOURCE_NO_SOURCE);
	}
}

void captouch_sim_init(struct captouch_match_entry *match_table, sdd_gesture_match_t *supported)
{
	if (match_table == NULL) {
		if (supported->zonem) {
			bb_log(BB_MOD_SDD, BB_LVL_DEBUG, "using default zonem captouch sim table");
			captouch_match_table = captouch_default_zonem_table;
		} else {
			bb_log(BB_MOD_SDD, BB_LVL_DEBUG, "using default captouch sim table");
			captouch_match_table = captouch_default_table;
		}
	} else {
		bb_log(BB_MOD_SDD, BB_LVL_WARNING, "optional captouch sim table not implemented, aborted");
		captouch_match_table = captouch_empty_table;
	}
	INIT_DELAYED_WORK(&captouch_work, captouch_sim);
}

static inline int captouch_sim_cmd_match(char *cmd, struct captouch_match_entry *cme)
{
	return (strncmp(cmd, cme->keyword, strlen(cme->keyword)) == 0);
}

static unsigned long captouch_sim_get_duration(char *param)
{
	u32 uval = 0;
	u32 max_duration = 1000 * 10;
	u32 duration = 0;
	int error;

	if (*param == '=') {
		if ((error = kstrtouint(param+1, 10, &uval))) {
			bb_log(BB_MOD_SDD, BB_LVL_WARNING, "unable to parse string to uint using kstrtouint, default to 0ms");
			return duration;
		}

		if (uval < max_duration) {
			duration = uval;
		} else {
			bb_log(BB_MOD_SDD, BB_LVL_WARNING, "duration of %ums exceeds max of %ums, default to 0ms",
				   uval, max_duration);
		}
	}
	return duration;
}

int captouch_sim_process_cmd(char *cmd)
{
	struct captouch_match_entry *cme;

	if (captouch_active_entry != NULL) {
		bb_log(BB_MOD_SDD, BB_LVL_WARNING, "simulated button event already in progress");
		return 0;
	}

	cme = captouch_match_table;
	while (cme->keyword != NULL) {
		if (captouch_sim_cmd_match(cmd, cme)) {
			struct captouch_sequence_entry *cse = cme->sequence;
			if (cse == NULL) {
				u32 duration = captouch_sim_get_duration(cmd+strlen(cme->keyword));
				captouch_msec_interval = (duration > SGE_PRESS_HOLD_TIME) ? SGE_PRESS_HOLD_TIME : duration;
				captouch_msec_remaining = duration;
				captouch_active_entry = cme;
				sdd_set_grr(cme->source);
				event_queue_send_event(cme->source, EVTINFO_PRESSED);
				schedule_delayed_work(&captouch_work, msecs_to_jiffies(captouch_msec_interval));
			} else {
				while (cse->source != EVTSOURCE_NO_SOURCE) {
					event_queue_send_event(cse->source, cse->info);
					msleep(cse->post_event_delay);
					cse++;
				}
			}
			return 1;
		}
		cme++;
	}
	return 0;
}

void captouch_sim_print_cmds(void)
{
	struct captouch_match_entry *cme = captouch_match_table;
	while (cme->keyword != NULL) {
		bb_log(BB_MOD_SDD, BB_LVL_WARNING, "  %s", cme->keyword);
		cme++;
	}
}
